/** @odoo-module **/

import { patch } from '@web/core/utils/patch'

import { TimeOffCalendarCommonRenderer } from '@hr_holidays/views/calendar/common/calendar_common_renderer'
import { onWillStart } from '@odoo/owl'
import { useService } from '@web/core/utils/hooks'
import { serializeDate } from '@web/core/l10n/dates'

patch(
  TimeOffCalendarCommonRenderer.prototype,
  {
    /**
     * Adding onWillStart to load data from the holidays.calendar model to get holidays, non-working days and rescheduled working days.
     */
    setup() {
      super.setup(...arguments)
      this.orm = useService('orm')
      onWillStart(async () => {
        const rangeStart = serializeDate(
          this.props.model.rangeStart,
          'datetime'
        )
        const rangeEnd = serializeDate(this.props.model.rangeEnd, 'datetime')
        const [holidaysList, nonWorkingDaysList, rescheduledWorkDaysList] =
          await this.orm.call('holidays.calendar', 'get_all_holidays', [
            rangeStart,
            rangeEnd
          ])
        this.holidays = holidaysList
        this.nonWorkingDays = nonWorkingDaysList
        this.rescheduledWorkDays = rescheduledWorkDaysList
      })
    },

    /**
     * Redefining onDayRender, adding additional functionality to it to display holidays, non-working days and postponed working days.
     */
    onDayRender(info) {
      const date = luxon.DateTime.fromJSDate(info.date).toISODate();
      if (this.props.model.unusualDays.includes(date)) {
        info.el.classList.add("o_calendar_disabled");
      }
      // New code
      if (this.nonWorkingDays.includes(date)) {
        info.el.classList.add('o_calendar_disabled')
      }
      const element = document.querySelector(
        `.fc-content-skeleton [data-date=${CSS.escape(date)}]`
      )
      if (element && element.classList.contains('fc-today')) {
        element.classList.remove('fc-today')
        element.classList.add('fc-today-day-number-month')
      }
      if (this.holidays.includes(date)) {
        if (element) {
          const day_number = element.querySelector('.fc-day-number')
          day_number.classList.add('hr_holiday_calendar')
          info.el.classList.add("o_calendar_disabled");
        } else {
          const days_in_week_element = document.querySelector(`.fc-head-container [data-date=${CSS.escape(date)}]`)
          info.el.classList.add("o_calendar_disabled");
          days_in_week_element.classList.add('o_holidays_week_days')
        }
      }
      if (this.rescheduledWorkDays.includes(date)) {
        info.el.classList.remove('o_calendar_disabled')
      }
    }
  }
)
